aboutsummaryrefslogtreecommitdiffstatshomepage
path: root/man.c
diff options
context:
space:
mode:
authorIngo Schwarze <schwarze@openbsd.org>2014-11-19 03:08:17 +0000
committerIngo Schwarze <schwarze@openbsd.org>2014-11-19 03:08:17 +0000
commitd447db580f83836c66e1273a0c4ccbf7bbc939eb (patch)
treee3ed791538e2e22d40928364ba431ae159bbaf98 /man.c
parent2eb6f6bbb8ff74e47725a337d3eaab3f1ffa44f0 (diff)
downloadmandoc-d447db580f83836c66e1273a0c4ccbf7bbc939eb.tar.gz
mandoc-d447db580f83836c66e1273a0c4ccbf7bbc939eb.tar.zst
mandoc-d447db580f83836c66e1273a0c4ccbf7bbc939eb.zip
Escape sequences terminate high-level macro names, and when doing so,
they are ignored, just in the same way as for request names and for low-level macro names. This also cures a warning in the pod2man(1) preamble.
Diffstat (limited to 'man.c')
-rw-r--r--man.c29
1 files changed, 22 insertions, 7 deletions
diff --git a/man.c b/man.c
index dedf62e7..55f9522f 100644
--- a/man.c
+++ b/man.c
@@ -1,4 +1,4 @@
-/* $Id: man.c,v 1.142 2014/11/03 23:18:39 schwarze Exp $ */
+/* $Id: man.c,v 1.143 2014/11/19 03:08:17 schwarze Exp $ */
/*
* Copyright (c) 2008, 2009, 2010, 2011 Kristaps Dzonsons <kristaps@bsd.lv>
* Copyright (c) 2013, 2014 Ingo Schwarze <schwarze@openbsd.org>
@@ -483,35 +483,50 @@ man_ptext(struct man *man, int line, char *buf, int offs)
static int
man_pmacro(struct man *man, int ln, char *buf, int offs)
{
- char mac[5];
struct man_node *n;
+ const char *cp;
enum mant tok;
int i, ppos;
int bline;
+ char mac[5];
ppos = offs;
/*
* Copy the first word into a nil-terminated buffer.
* Stop copying when a tab, space, or eoln is encountered.
*/
i = 0;
- while (i < 4 && '\0' != buf[offs] && ' ' != buf[offs] &&
- '\t' != buf[offs])
+ while (i < 4 && strchr(" \t\\", buf[offs]) == NULL)
mac[i++] = buf[offs++];
mac[i] = '\0';
tok = (i > 0 && i < 4) ? man_hash_find(mac) : MAN_MAX;
- if (MAN_MAX == tok) {
+ if (tok == MAN_MAX) {
mandoc_msg(MANDOCERR_MACRO, man->parse,
ln, ppos, buf + ppos - 1);
return(1);
}
- /* The macro is sane. Jump to the next word. */
+ /* Skip a leading escape sequence or tab. */
+
+ switch (buf[offs]) {
+ case '\\':
+ cp = buf + offs + 1;
+ mandoc_escape(&cp, NULL, NULL);
+ offs = cp - buf;
+ break;
+ case '\t':
+ offs++;
+ break;
+ default:
+ break;
+ }
+
+ /* Jump to the next non-whitespace word. */
while (buf[offs] && ' ' == buf[offs])
offs++;
@@ -483,35 +483,50 @@ man_ptext(struct man *man, int line, char *buf, int offs)
/*
* Copy the first word into a nil-terminated buffer.
- * Stop copying when a tab, space, or eoln is encountered.
+ * Stop when a space, tab, escape, or eoln is encountered.
*/
i = 0;